      MODULE DefBaseGrid


!     use constants
!     use scatterer,ONLY:RefractiveIndex,Suski,InParticle,
      use scatterer,ONLY:InParticle,
     & subgrains,grain,grainclass,grainclasses
      use special,ONLY:GetBessels,VecSphHarms

      IMPLICIT NONE

!  -------------------------------------------
!       PUBLIC DefineBaseGrid        !   Initialization, to be called once
!  --------------------------------------------

   
!
!     cdim: total number of cells
!     gdim: number of non-redundant cells
!
!
      INTEGER,SAVE::cdim,gdim,gn1,gn2,gn3,sgn

      LOGICAL,SAVE,DIMENSION(:,:,:,:),ALLOCATABLE::gridmask,
     & gridmask2,gridmask3  ! (x,y,z,cell)

      COMPLEX*16,SAVE,DIMENSION(:,:,:,:,:,:),
     & ALLOCATABLE::baski,baski2,baski3
! Later this may be changed to packed format to save space, 
! now we try to keep it simple and stupid.

      REAL*4,SAVE,DIMENSION(:,:,:,:,:),
     & ALLOCATABLE::posi,posi2,posi3,grs,grs2,grs3
      REAL*4,SAVE,DIMENSION(:,:,:,:),
     & ALLOCATABLE::gridvol,gridvol2,gridvol3

      COMPLEX*8,SAVE,DIMENSION(:,:,:,:),
     & ALLOCATABLE::invsuski !,invsuski2,invsuski3            
!     REAL*4,SAVE,DIMENSION(:,:,:,:),
!    & ALLOCATABLE::permis,chiralis,nonreciprocis  ! for future use


      INTEGER,PARAMETER::free=0,inside=1,surface=2,edge=3,corner=4
       TYPE celltype
       INTEGER::first,last,exporder,J,ingrain,gridpoints,
     &  gridpointspCell,grid
       REAL*4,DIMENSION(3)::center
       REAL*4,DIMENSION(3,3)::rotation
       REAL*4 volume,size,spacing
       LOGICAL homogeneous,magnetic,chiral,redundant
       COMPLEX*8 cellsuskis,meanindex
       REAL*4 cellpermis,cellchiralis,cellnonreciprocis
       INTEGER status,first_subcell,last_subcell
      END TYPE celltype


      TYPE(celltype),SAVE,DIMENSION(:),ALLOCATABLE::cell

      REAL,SAVE::reqaccuracy



      INTEGER,DIMENSION(0:11)::unorder=
     & (/3,6,11,16,23,30,39,48,59,70,83,96/)


! --------------------------------------------------------------

	CONTAINS 

!     ****************************************************************
      subroutine DefineBaseGrid(dim)
!     *******************************************************
      USE Timing,ONLY:CPUtime
      INTEGER,INTENT(out)::dim
      INTEGER  c,cc,g,i,c1,c2,c3,celldivpSubgr,subgrain,oldgrain! indices
      INTEGER  gridpointspCell,minJ,maxJ,valiJ,hits,tothits,
     & cellsatstart,recmaxJ,recminJ,recdiv,reccells
      INTEGER maxgrids
      REAL*4 totvol,am2,minminscale

       minminscale=minval(grainclass(1:grainclasses)%minscale)

       WRITE(*,*) 'Please, specify the accuracy requested:'
       WRITE(*,*) '[start with low accuracy and proceed..]'
       READ(15,*) reqaccuracy
       WRITE(*,*) reqaccuracy
       IF (grainclass(1)%maxscale*sqrt(abs(grainclass(1)%refindx)**2+
     &  1.0/grainclass(1)%minscale**2) < 0.5 ) THEN
	 reccells=1
       ELSE
	 reccells=int(1.0/(3*reqaccuracy)**0.33*
     &  (0.6*maxval(grainclass(1)%boxsize)*
     &  sqrt(abs(grainclass(1)%refindx)**2+
     &  1.0/grainclass(1)%minscale**2)))
       ENDIF
       WRITE(*,*) 'Number of cell divisions per subgrain per '
       WRITE(*,*) ' direction? [',reccells,']'
       celldivpSubgr=reccells
!      READ(*,*) celldivpSubgr 
       IF (celldivpSubgr<=0) celldivpSubgr=reccells
       WRITE(*,*) celldivpSubgr 
       recdiv=ceiling(maxval(grainclass(1)%boxsize)/celldivpSubgr* 
     &    sqrt(abs(grainclass(1)%refindx)**2+1.0
     &    /grainclass(1)%minscale**2)/ 
     &    sqrt(24*reqaccuracy)*1.25)
!           ceiling(1.0/reqaccuracy**0.66666/celldivpSubgr))
       recdiv=nint(recdiv*0.501)*2
       WRITE(*,*) 'Number of grid points per direction'
       WRITE(*,'(A,I3,A)')' per cell? [ >=',recdiv ,
     &     ',higher if oddly shaped]'
       gridpointspCell=recdiv
!      READ(*,*) gridpointspCell
       IF (gridpointspCell<=0) gridpointspCell=recdiv
       WRITE(*,*) gridpointspCell
       IF (celldivpSubgr==1) THEN
         am2=sqrt((abs(grainclass(1)%refindx)*
     &    grainclass(1)%maxscale)**2+
     &   (grainclass(1)%maxscale/grainclass(1)%minscale)**2)
       ELSE
	  am2=maxval(grainclass(1)%boxsize)/celldivpSubgr*0.7*  
     &    !(1.0-1.0/gridpointspCell)* 
     & sqrt(abs(grainclass(1)%refindx)**2+(1.0/grainclass(1)%minscale-
     & 1.0/grainclass(1)%maxscale)**2 ) 
       ENDIF

! One should fine something better.
       IF (am2*(1.0+am2/2.0+am2**2/6)<reqaccuracy) THEN
            recminJ=0
	ELSEIF (am2**2/2*(1.0+am2/3.0+am2**2/12.0)<reqaccuracy) THEN
               recminJ=1
	ELSEIF (am2**3/(2*3)*(1.0+am2/4.0+am2**2/(4*5))<reqaccuracy)
     &     THEN
               recminJ=2
	ELSEIF (am2**4/(2*3*4)*(1.0+am2/5+am2**2/(5*6)+
     &           am2**3/(7*6*5))<reqaccuracy) THEN
               recminJ=3
	ELSEIF (am2**5/(2*3*4*5)*(1.0+am2/6+am2**2/(7*6))<reqaccuracy)
     &     THEN
               recminJ=4
	ELSEIF (am2**6/(2*3*4*5*6)*(1.0+am2/7+am2**2/8)<reqaccuracy)
     &     THEN
               recminJ=5
	ELSEIF (am2**7/(2*3*4*5*6*7)*(1.0+am2/8+am2**2/9)<reqaccuracy)
     &     THEN
               recminJ=6
	ELSE
               recminJ=99
       ENDIF
       IF (am2<reqaccuracy*(recdiv*1.0/gridpointspCell)**2) THEN
              recmaxJ=1
	 ELSEIF (am2**2/2<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2) THEN
              recmaxJ=2
	 ELSEIF (am2**3/(2*3)<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2) THEN
              recmaxJ=3
	 ELSEIF (am2**4/(2*3*4)<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2) THEN
              recmaxJ=4
	 ELSEIF (am2**5/(2*3*4*5)<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2) 
     &     THEN
              recmaxJ=5
	 ELSEIF (am2**6/(2*3*4*5*6)<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2)
     &     THEN
              recmaxJ=6
	 ELSEIF (am2**7/(2*3*4*5*6*7)<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2)
     &     THEN
              recmaxJ=7
	 ELSEIF (am2**8/(2*3*4*5*6*7*8)<
     &     reqaccuracy*(recdiv*1.0/gridpointspCell)**2)
     &     THEN
              recmaxJ=8
	 ELSE
              recmaxJ=9
       ENDIF

      WRITE(*,*)'Expansion order (0=Edipole,1=E+Mdpol;2,3=qpol;
     & 4,5=opol)?'
      WRITE(*,'(A,I2,A,I2,A)') '[ >=',recminJ,'   <=',recmaxJ,']'
       minJ =recminJ
       READ(15,*)  minJ 
!      if (minJ<0) minJ=min(recMaxJ,recminJ)
!      minJ=min(recmaxJ,recminJ)
       maxJ=MinJ 
       WRITE(*,*) 'Fine tuning of the quadrature point?'
       WRITE(*,*) '  [',gridpointspCell*celldivpSubgr,']'
       sgn =gridpointspCell*celldivpSubgr
!      READ(*,*)  sgn 
       if (sgn<=0) sgn=gridpointspCell*celldivpSubgr
!
       CALL Factorize(gridpointspCell,gn1,gn2,gn3)
!
       WRITE(*,'(I3,I3,A2,I2,A1,I2,A1,I2,A4,I3,I3)') 
     &    celldivpSubgr,gridpointspCell,
     &    '(=',gn1,'*',gn2,'*',gn3,')   ',minJ,sgn
       WRITE(*,*) 
       WRITE(*,*) 'Thank You for your input !'
       WRITE(*,*) 'The computation begins. '
       WRITE(*,*) ' I wish You a lot of patience...'
       WRITE(*,*) '================================================='
       WRITE(*,*) 
!
       cellsatstart=celldivpSubgr**3*subgrains
       maxgrids=celldivpSubgr**3*grainclasses
       IF (maxgrids>80) maxgrids=40+maxgrids/2
       ALLOCATE(cell(0:cellsatstart-1))  
       ALLOCATE(gridmask(gridpointspCell,gridpointspCell,
     & gridpointspCell,min(cellsatstart,maxgrids)))  
       ALLOCATE(gridvol(gridpointspCell,gridpointspCell,
     & gridpointspCell,min(cellsatstart,maxgrids)))  
       ALLOCATE(posi(gridpointspCell,gridpointspCell,
     & gridpointspCell,3,min(cellsatstart,maxgrids)))  
       ALLOCATE(grs(gridpointspCell,gridpointspCell,
     & gridpointspCell,3,min(cellsatstart,maxgrids)))  
       ALLOCATE(invsuski(gridpointspCell,gridpointspCell,
     & gridpointspCell,min(cellsatstart,maxgrids)))  
       c=0;g=1;i=1;tothits=0;totvol=0.0


	grainloop: DO subgrain=1,subgrains
!--------
!    Look back if something similar is found.
	 DO oldgrain=1,subgrain-1  
	   IF (grain(subgrain)%class==grain(oldgrain)%class) THEN 
!    found:
	     DO cc=0,c-1
!               write(*,*)subgrain,oldgrain,c,cc,i
	       IF (oldgrain==cell(cc)%ingrain) THEN
		   cell(c)=cell(cc)
		   cell(c)%ingrain=subgrain
		   cell(c)%center=cell(cc)%center
     &              +grain(subgrain)%center-grain(oldgrain)%center
		   cell(c)%redundant=.true.
		   cell(c)%first=i; cell(c)%last=i+cell(c)%exporder-1
		   tothits=tothits+cell(cc)%gridpoints
		   totvol=totvol+cell(cc)%volume
		   i=i+cell(c)%exporder
		   c=c+1
		ENDIF
	     ENDDO
	     CYCLE grainloop
	   ENDIF
	 ENDDO
!		
!++ cell-loop over celldivpSubgr**3 +++++++++++++++++++++++++++++++++
!
	DO c1=1,celldivpSubgr
	 DO c2=1,celldivpSubgr
	  DO c3=1,celldivpSubgr
	  IF (g>maxgrids) THEN
	    WRITE(*,*) 'Sorry, but a hard limit of',maxgrids,
     &         ' different cells is reached.'
	    WRITE(*,*) 'Some programming may help'
	    STOP
	  ENDIF
!          write(*,*)'# ',subgrain,'(/',subgrains,')   [',c1,c2,c3,'] ' !,CPUtime()

	  cell(c)%size=
     &      maxval(grainclass(grain(subgrain)%class)%boxsize)
     &            /celldivpSubgr
	  cell(c)%spacing=cell(c)%size/gridpointspCell
	  cell(c)%center=grain(subgrain)%center+((/c1,c2,c3/)-
     &          0.5*(celldivpSubgr+1))*cell(c)%size
!        matmul( -..- , grain(subgrain)%rotation) 
	  cell(c)%ingrain=subgrain
	  cell(c)%cellsuskis=
     &    grainclass(grain(subgrain)%class)%refindx**2-REAL(1.0)
	  cell(c)%meanindex=grainclass(grain(subgrain)%class)%refindx
	  cell(c)%homogeneous=
     &    grainclass(grain(subgrain)%class)%homogeneous
	  cell(c)%magnetic=grainclass(grain(subgrain)%class)%magnetic
	  cell(c)%chiral=grainclass(grain(subgrain)%class)%chiral
	  cell(c)%grid=g
	  cell(c)%gridpointspCell=gridpointspCell
!      write(*,*)'refin',cell(c)%meanindex,cell(c)%cellsuskis
!
!  loop over grids per cell: generate grid for this cell
!
	  CALL GenerateGrid(gridmask(:,:,:,g),gridvol(:,:,:,g),
     &         posi(:,:,:,:,g),grs(:,:,:,:,g),invsuski(:,:,:,g),
     &         cell(c),hits,gridpointspCell,sgn)
!
!          write(*,*)'hits:',hits,' vol:',cell(c)%volume
	  IF (hits>0) THEN            
	    cell(c)%volume=sum(gridvol(:,:,:,g),MASK=gridmask(:,:,:,g))
	    cell(c)%exporder=0
	    cell(c)%J=0
! This point still needs rethinking
	    IF (hits==1) THEN
		 cell(c)%exporder=unorder(0)
		 cell(c)%J=0
	    ELSE IF (celldivpSubgr<=2) THEN
	      DO valiJ=0,maxJ
		 IF ((unorder(valiJ)>(hits*3) )) EXIT
		 cell(c)%exporder=unorder(valiJ)
		 cell(c)%J=valiJ
	      ENDDO
	    ELSE
              DO valiJ=0,maxJ
		 IF (unorder(valiJ)>(hits*3) ) EXIT
!                 IF (unorder(valiJ)> (3*gridpointspCell**3-unorder(maxJ+1)+
!    &               (unorder(maxJ+1)-3)*hits)/(gridpointspCell**3-1) ) EXIT
		 cell(c)%exporder=unorder(valiJ)
		 cell(c)%J=valiJ
	      ENDDO
	    ENDIF
	    IF (cell(c)%J/=maxJ) write(*,*)'order reduced:',maxJ,'->',
     &         cell(c)%J,c,hits,'/',gridpointspCell**3
	    cell(c)%first=i; cell(c)%last=i+cell(c)%exporder-1
	    cell(c)%gridpoints=hits;tothits=tothits+hits
	    totvol=totvol+cell(c)%volume
!
	    CALL RemoveRedundancy(c)
!
	    IF (.NOT.cell(c)%redundant) THEN
	       g=g+1   
	    ENDIF
!          write(*,*)'# ',subgrain,'(/',subgrains,') ',c,'= [',c1,c2,c3,'] ',
!    &        cell(c)%meanindex,CPUtime()
	    i=i+cell(c)%exporder
	    c=c+1
	  ENDIF
	 ENDDO
	ENDDO
       ENDDO
!
!    End of cell-loop over celldivpSubgr**3
!
       ENDDO grainloop
!!-----
       dim=i-1
       cdim=c-1 
       gdim=g-1  
       ALLOCATE(baski(gridpointspCell,gridpointspCell,gridpointspCell,3,
     &  unorder(maxJ),gdim))
       ALLOCATE(gridmask2(gn2*gn3,gn2*gn3,gn2*gn3,gdim))
       ALLOCATE(posi2(gn2*gn3,gn2*gn3,gn2*gn3,3,gdim))  
       ALLOCATE(grs2(gn2*gn3,gn2*gn3,gn2*gn3,3,gdim))  
       ALLOCATE(gridvol2(gn2*gn3,gn2*gn3,gn2*gn3,gdim))  
       ALLOCATE(baski2(gn2*gn3,gn2*gn3,gn2*gn3,3,unorder(maxJ),gdim))
!       ALLOCATE(invsuski2(gn2*gn3,gn2*gn3,gn2*gn3,gdim))  
       ALLOCATE(gridmask3(gn3,gn3,gn3,gdim))  
       ALLOCATE(posi3(gn3,gn3,gn3,3,gdim))  
       ALLOCATE(grs3(gn3,gn3,gn3,3,gdim))  
       ALLOCATE(gridvol3(gn3,gn3,gn3,gdim))  
       ALLOCATE(baski3(gn3,gn3,gn3,3,unorder(maxJ),gdim))
!       ALLOCATE(invsuski3(gn3,gn3,gn3,gdim))  
!
       DO c=0,cdim         
	 g=cell(c)%grid
!
	 IF (.not.cell(c)%redundant) 
!
     &    CALL EvaluateBaseFunctions(gridmask(:,:,:,g),
     &    baski(:,:,:,:,:,g),gridvol(:,:,:,g),posi(:,:,:,:,g),
     &    grs(:,:,:,:,g),
     &    gridmask2(:,:,:,g), baski2(:,:,:,:,:,g),gridvol2(:,:,:,g),
     &    posi2(:,:,:,:,g),grs2(:,:,:,:,g),
     &    gridmask3(:,:,:,g), baski3(:,:,:,:,:,g),gridvol3(:,:,:,g),
     &    posi3(:,:,:,:,g),grs3(:,:,:,:,g),c,gridpointspCell,gn2,gn3,
     &    unorder(maxJ))
!
!          write(*,*)c, '###################################'
!          DO c1=1,cell(c)%gridpointspCell
!          DO c2=1,cell(c)%gridpointspCell
!          DO c3=1,cell(c)%gridpointspCell
!           write(*,*)gridmask(c1,c2,c3,cell(c)%grid),c1,c2,c3
!           write(*,*)sum(abs(baski(:,:,c1,c2,c3,cell(c)%grid))),
!     &        gridvol(c1,c2,c3,cell(c)%grid)
!          ENDDO;ENDDO ; ENDDO

       ENDDO       
	       
       write(*,*)'total order of expansion:',dim
       write(*,*)'  number of cells:',cdim+1
       write(*,*)'  number of grid points:',tothits 
       write(*,*)' Number of different grids:',gdim
       write(*,*)' Total volume:',totvol
       IF (dim>1000) write(*,*)'WARNING: dimension high,
     & solving can take time'
       IF (dim>2000) then
                     write(*,*)'WARNING: looking for troubles - '
                     write(*,*)' dimension very high'
       ENDIF
!
!      STOP
       END Subroutine DefineBaseGrid


! **************************************
       SUBROUTINE RemoveRedundancy(c)
! *************************************
       INTEGER,INTENT(in)::c
       INTEGER cc

	cell(c)%redundant=.false.
       DO cc=0,c-1        ! remove redundant grid structure
       IF (ALL(gridmask(:,:,:,cell(c)%grid).EQV.
     &   gridmask(:,:,:,cell(cc)%grid)).AND.
     &   ALL(posi(:,:,:,:,cell(c)%grid)==
     &    posi(:,:,:,:,cell(cc)%grid)).AND.
     &   (grain(cell(c)%ingrain)%class==
     &    grain(cell(cc)%ingrain)%class).AND.
     &   (cell(c)%cellsuskis==cell(cc)%cellsuskis))  THEN
		 cell(c)%redundant=.true.
		 cell(c)%grid=cell(cc)%grid
!                cell(c)%rotation=???           
		 write(*,*)'redundancy removed',c,'=>',cc
		 EXIT
	ENDIF
        ENDDO
	END SUBROUTINE RemoveRedundancy


! ************************************************************************
	SUBROUTINE GenerateGrid(mask,volu,posi,grs,invski,celli,hits,
     &    gridpointspCell,sgn)
! *******************************************************************************
      TYPE(CELLTYPE),INTENT(in)::celli
      INTEGER,INTENT(out)::hits
      INTEGER,INTENT(in)::sgn,gridpointspCell
      LOGICAL,INTENT(out)
     & ::mask(celli%gridpointspCell,celli%gridpointspCell,
     &        celli%gridpointspCell)
      REAL*4,INTENT(out),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell,3)
     & ::posi,grs
      REAL*4,INTENT(out),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell)::volu
      COMPLEX*8,INTENT(out),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell)
     & ::invski
      integer g1,g2,g3,sg1,sg2,sg3,subhits
      REAL*4,DIMENSION(3):: apuposi,apuposi2,posix
     
      hits=0
      DO g1=1,gridpointspCell
        DO g2=1,gridpointspCell;  DO g3=1,gridpointspCell
      subhits=0;posix=0.0
      apuposi=celli%spacing*((/g1,g2,g3/)-0.5*(gridpointspCell+1))
      DO sg1=1,sgn
       DO sg2=1,sgn
        DO sg3=1,sgn
	   apuposi2=apuposi+
     &             celli%spacing/sgn*((/sg1,sg2,sg3/)-0.5*(sgn+1))
	   IF (InParticle(apuposi2+celli%center,grain(celli%ingrain)))
     &        THEN
		 subhits=subhits+1
		 posix=posix+apuposi2
	   ENDIF
      ENDDO;ENDDO;ENDDO
      IF ( (subhits*1.0/sgn**3)>0.9) THEN
	 mask(g1,g2,g3)=.TRUE.
	 posi(g1,g2,g3,:)=apuposi
	 grs(g1,g2,g3,:)=celli%spacing
	 volu(g1,g2,g3)=celli%spacing**3
	 invski(g1,g2,g3)=REAL(1.0)/celli%cellsuskis
	 hits=hits+1
      ELSE IF ( (subhits*1.0/sgn**3)>0.1) THEN
	 mask(g1,g2,g3)=.TRUE.
	 posi(g1,g2,g3,:)=posix/subhits
	 grs(g1,g2,g3,:)=celli%spacing -REAL(2.0)*
     &                   abs(apuposi-posi(g1,g2,g3,:))
	 volu(g1,g2,g3)=(celli%spacing/sgn)**3*subhits
	 IF (product(grs(g1,g2,g3,:)) >=  volu(g1,g2,g3)) THEN
	    grs(g1,g2,g3,:)=grs(g1,g2,g3,:)*
     &       volu(g1,g2,g3)/product(grs(g1,g2,g3,:))
	 ELSE
	    volu(g1,g2,g3)=product(grs(g1,g2,g3,:))
	 ENDIF
	 invski(g1,g2,g3)=REAL(1.0)/celli%cellsuskis
	 hits=hits+1
      ELSE
	 mask(g1,g2,g3)=.FALSE.
	 posi(g1,g2,g3,:)=0.0;volu(g1,g2,g3)=Tiny(REAL(1.0))*10
         invski(g1,g2,g3)=Huge(REAL(1.0))*0.1
	 grs(g1,g2,g3,:)=0.0
      ENDIF
      ENDDO;ENDDO;ENDDO
      END SUBROUTINE GenerateGrid


! ********************************************************************
	SUBROUTINE EvaluateBaseFunctions(gridmask,baski,gvol,posi,
     &    grs,gridmask2,baski2,gvol2,posi2,grs2,
     &    gridmask3,baski3,gvol3,posi3,grs3,c,gridpointspCell,gn2,
     &    gn3,ed)
! **************************************************************************

!     INTRINSIC REAL,DBLE
      INTEGER,INTENT(in)::c,gridpointspCell,gn2,gn3,ed
      COMPLEX*16,INTENT(out),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell,3,ed)
     & ::baski
      COMPLEX*16,INTENT(out),
     & DIMENSION(gn2*gn3,gn2*gn3,gn2*gn3,3,ed)::baski2
      COMPLEX*16,INTENT(out),
     & DIMENSION(gn3,gn3,gn3,3,ed)::baski3
      REAL*8,DIMENSION(ed)::BB !,BB2,BB3
      REAL*4,INTENT(in),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell,3)
     & ::posi,grs
      REAL*4,INTENT(out),
     & DIMENSION(gn2*gn3,gn2*gn3,gn2*gn3,3)::posi2,grs2
      REAL*4,INTENT(out),
     & DIMENSION(gn3,gn3,gn3,3)::posi3,grs3
      REAL*4,INTENT(in),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell)::gvol
      REAL*4,INTENT(out),
     & DIMENSION(gn2*gn3,gn2*gn3,gn2*gn3)::gvol2
      REAL*4,INTENT(out),DIMENSION(gn3,gn3,gn3)::gvol3
      LOGICAL,INTENT(in),
     & DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell)
     & ::gridmask
      LOGICAL,INTENT(out),
     &    DIMENSION(gn2*gn3,gn2*gn3,gn2*gn3)::gridmask2
      LOGICAL,INTENT(out),DIMENSION(gn3,gn3,gn3)::gridmask3
      integer g1,g2,g3,gg1,gg2,gg3,ggg1,ggg2,ggg3,i,j  !,hits2,hits1
      REAL*4,DIMENSION(3)::centerofmass
      LOGICAL,DIMENSION(gridpointspCell,gridpointspCell,gridpointspCell)
     &    ::testmask
  
      testmask=.false.
      baski=DBLE(0.0);baski2=0.0;baski3=0.0
      posi2=0.0;posi3=0.0
      grs2=0.0;grs3=0.0
      gvol2=0.0;gvol3=0.0
      gridmask2=.FALSE.;gridmask3=.FALSE.
      centerofmass(1)=sum(posi(:,:,:,1),MASK=gridmask)/count(gridmask)
      centerofmass(2)=sum(posi(:,:,:,2),MASK=gridmask)/count(gridmask)
      centerofmass(3)=sum(posi(:,:,:,3),MASK=gridmask)/count(gridmask)
!   write(*,*)'cm',centerofmass
      DO ggg1=1,gn3;  DO ggg2=1,gn3;  DO ggg3=1,gn3
!      hits2=0
      DO gg1=(ggg1-1)*gn2+1,ggg1*gn2; DO gg2=(ggg2-1)*gn2+1,ggg2*gn2
      DO gg3=(ggg3-1)*gn2+1,ggg3*gn2 
!       hits1=0
       DO g1=(gg1-1)*gn1+1,gg1*gn1; DO g2=(gg2-1)*gn1+1,gg2*gn1
       DO g3=(gg3-1)*gn1+1,gg3*gn1 
	  IF (testmask(g1,g2,g3)) write(*,*)'evaluation sick',g1,g2,g3
	  testmask(g1,g2,g3)=.true.
	  IF (gridmask(g1,g2,g3)) THEN
!
	      CALL GetBasis(baski(g1,g2,g3,:,1:cell(c)%exporder),
!
     &         cell(c)%meanindex,-centerofmass+posi(g1,g2,g3,:),
     &         cell(c)%exporder,cell(c)%J)
	      baski(g1,g2,g3,:,:)=cell(c)%cellsuskis*
     &             baski(g1,g2,g3,:,:)*gvol(g1,g2,g3)
	      baski2(gg1,gg2,gg3,:,:)=baski2(gg1,gg2,gg3,:,:)+
     &             baski(g1,g2,g3,:,:)
	      posi2(gg1,gg2,gg3,:)=posi2(gg1,gg2,gg3,:)+
     &             posi(g1,g2,g3,:)*gvol(g1,g2,g3)
	      posi3(ggg1,ggg2,ggg3,:)=posi3(ggg1,ggg2,ggg3,:)+
     &             posi(g1,g2,g3,:)*gvol(g1,g2,g3)
	      grs2(gg1,gg2,gg3,:)=grs2(gg1,gg2,gg3,:)+
     &             grs(g1,g2,g3,:)*gvol(g1,g2,g3)
	      grs3(ggg1,ggg2,ggg3,:)=grs3(ggg1,ggg2,ggg3,:)+
     &             grs(g1,g2,g3,:)*gvol(g1,g2,g3)
	      gvol2(gg1,gg2,gg3)=gvol2(gg1,gg2,gg3)+gvol(g1,g2,g3)
	      gridmask2(gg1,gg2,gg3)=.TRUE.
!              hits1=hits1+1
!              hits2=hits2+1
	  ENDIF
       ENDDO;ENDDO;ENDDO
       IF (gridmask2(gg1,gg2,gg3)) THEN
	 posi2(gg1,gg2,gg3,:)=posi2(gg1,gg2,gg3,:)/gvol2(gg1,gg2,gg3)
	 grs2(gg1,gg2,gg3,:)=grs2(gg1,gg2,gg3,:)*g
     &    vol2(gg1,gg2,gg3)/product(grs2(gg1,gg2,gg3,:))
	 baski3(ggg1,ggg2,ggg3,:,:)=baski3(ggg1,ggg2,ggg3,:,:)+
     &    baski2(gg1,gg2,gg3,:,:)
	 gvol3(ggg1,ggg2,ggg3)=gvol3(ggg1,ggg2,ggg3)+gvol2(gg1,gg2,gg3)
	 gridmask3(ggg1,ggg2,ggg3)=.TRUE.
       ENDIF
       ENDDO;ENDDO;ENDDO
      IF (gridmask3(ggg1,ggg2,ggg3)) posi3(ggg1,ggg2,ggg3,:)=
     &    posi3(ggg1,ggg2,ggg3,:)/gvol3(ggg1,ggg2,ggg3)
	  IF (gridmask3(ggg1,ggg2,ggg3))
     &     grs3(ggg1,ggg2,ggg3,:)=grs3(ggg1,ggg2,ggg3,:)*
     &     gvol3(ggg1,ggg2,ggg3)/product(grs3(ggg1,ggg2,ggg3,:))
	  ENDDO;ENDDO;ENDDO
	  IF (.NOT.ALL(testmask)) write(*,*)'evaluation still sick',
     &       testmask
	  DO j=1,cell(c)%exporder
	    BB (j)=sum(sum(real(conjg(baski (:,:,:,:,j))*
     &        baski (:,:,:,:,j)),dim=4)/gvol ,mask=gridmask)
!           BB2(j)=sum(sum(conjg(baski2(:,:,:,:,j))*
!    &        baski2(:,:,:,:,j),dim=4)/gvol2,mask=gridmask2)
!           BB3(j)=sum(sum(conjg(baski3(:,:,:,:,j))*
!    &        baski3(:,:,:,:,j),dim=4)/gvol3,mask=gridmask3)
	  ENDDO
!  write(*,*)'baski',baski
!  write(*,*)'gridvol',gvol
!  write(*,*)'gridmask',gridmask
!  write(*,*)'BB',BB
!  write(*,*)'eo',c,cell(c)%exporder,ed,cell(c)%cellsuskis
      DO i=1,3;  DO j=1,cell(c)%exporder
      WHERE (gridmask)  baski (:,:,:,i,j)=baski (:,:,:,i,j)/sqrt(BB(j))
      WHERE (gridmask2) baski2(:,:,:,i,j)=baski2(:,:,:,i,j)/sqrt(BB(j))
      WHERE (gridmask3) baski3(:,:,:,i,j)=baski3(:,:,:,i,j)/sqrt(BB(j))
      ENDDO; ENDDO

	END SUBROUTINE EvaluateBaseFunctions


! ******************************************************************
      SUBROUTINE GetBasis(basis,refindex,r0,exporder,maxJ)
! ******************************************************************
      INTEGER,INTENT(in)::maxJ,exporder
      COMPLEX*16,INTENT(out),DIMENSION(3,exporder)::basis
      COMPLEX*8,INTENT(in)::refindex
      REAL*4,DIMENSION(3),INTENT(in):: r0
      integer i
      INTEGER,DIMENSION(:),ALLOCATABLE::l,m
      LOGICAL,DIMENSION(:),ALLOCATABLE::magnetic
      COMPLEX*16,DIMENSION(0:maxJ/2+2)::Bessel
      REAL*4 RR,rxy,costheta,sintheta
      COMPLEX*8 efi
      COMPLEX*16,DIMENSION(3,maxj/2+1,-1:1,-(maxJ/2+1):maxJ/2+1)
     &    ::vectorSphericalHarmonics
!         note, the second index is relative to the first, i.e l'=l+j
      LOGICAL FIRSTbasis
      REAL*4,SAVE::sqrt1per3,sqrt2per3

      FIRSTbasis=.TRUE.

	IF (FIRSTbasis) THEN
	  sqrt1per3=sqrt(1.0/3.0)
	  sqrt2per3=sqrt(2.0/3.0)
	  FIRSTbasis=.FALSE.
	ENDIF
       rr=sqrt(dot_product(r0,r0))
       rxy=sqrt(r0(1)**2+r0(2)**2)
       IF (rxy<Epsilon(rxy)*99.9) THEN
          efi=1.0 
       ELSE
          efi=cmplx(r0(1),r0(2))/rxy
       ENDIF
       IF (rr<Epsilon(rr)*99.9) THEN
          costheta=0.0
          sintheta=REAL(1.0)
       ELSE; costheta=r0(3)/rr;sintheta=rxy/rr;ENDIF
       IF (abs((costheta**2+sintheta**2)-REAL(1.0))>
     &       10*Epsilon(costheta))
     &     write(*,*)'cssn',costheta,sintheta
       IF (exporder/=unorder(MaxJ)) STOP 'GetBasis munii'
       ALLOCATE(l(exporder),m(exporder),magnetic(exporder))
       IF (MaxJ==0) THEN
	  l(1:3)=1;m(1:3)=(/-1,0,1/)
!
	  CALL getBessels(rr*refindex,0,Bessel)
	  CALL VecSphHarms(costheta,sintheta,efi,1,
     &                      vectorSphericalHarmonics)
!
!         write(*,*)'Y',vectorsphericalharmonics
	  DO i=1,exporder
	     Basis(:,i)=Bessel(0)*VectorSphericalHarmonics(:,1,-1,m(i))
	  ENDDO
       ELSE
	 l(1:6)=1;magnetic(1:3)=.FALSE.
         magnetic(4:6)=.TRUE.;m(1:6)=(/-1,0,1,-1,0,1/)
	 IF (maxJ>1) THEN
	  l(7:11)=2;magnetic(7:11)=.FALSE.
          m(7:11)=(/-2,-1,0,1,2/)
	  IF (maxJ>2) THEN
	   l(12:16)=2;magnetic(12:16)=.TRUE.
           m(12:16)=(/-2,-1,0,1,2/)
	   IF (maxJ>3) THEN
	    magnetic(17:23)=.FALSE.; l(17:23)=3
            m(17:23)=(/-3,-2,-1,0,1,2,3/)
	    IF (maxJ>4) THEN 
	      magnetic(24:30)=.TRUE.; l(24:30)=3
              m(24:30)=(/-3,-2,-1,0,1,2,3/)
	      IF (maxJ>5) THEN 
		magnetic(31:39)=.FALSE.; l(31:39)=4
                m(31:39)=(/-4,-3,-2,-1,0,1,2,3,4/)
		IF (maxJ>6) THEN 
		  magnetic(40:48)=.TRUE.; l(40:48)=4
                  m(40:48)=(/-4,-3,-2,-1,0,1,2,3,4/)
		  IF (maxJ>7) THEN 
		     magnetic(49:59)=.FALSE.; l(49:59)=5
                     m(49:59)=(/-5,-4,-3,-2,-1,0,1,2,3,4,5/)
		     IF (maxJ>8) THEN 
		       magnetic(60:70)=.TRUE.; l(60:70)=5
                       m(60:70)=(/-5,-4,-3,-2,-1,0,1,2,3,4,5/)
		       IF (maxJ>9) STOP
		     ENDIF    
		  ENDIF    
		ENDIF    
	    ENDIF    
	    ENDIF    
	   ENDIF    
	  ENDIF    
	 ENDIF
!
	 CALL getBessels(rr*refindex,maxJ/2+2,Bessel)
!         write(*,*)rr,Bessel
	 CALL VecSphHarms(costheta,sintheta,efi,maxJ/2+1,
     &                         vectorSphericalHarmonics)
!
	 DO i=1,exporder
	  IF (magnetic(i)) THEN
	     Basis(:,i)=Bessel(l(i))*
     &                  VectorSphericalHarmonics(:,l(i),0,m(i))
	  ELSE !'electric'
	   IF (l(i)==1) THEN
	      Basis(:,i) = sqrt(DBLE(1.0)/DBLE(3.0))*Bessel(2)*
     &               VectorSphericalHarmonics(:,1,+1,m(i))
     &      -sqrt(DBLE(2.0)/DBLE(3.0))*Bessel(0)*
     &     VectorSphericalHarmonics(:,1,-1,m(i))  
	   ELSE
	      Basis(:,i)=sqrt(l(i)/(2*l(i)+DBLE(1.0)))*Bessel(l(i)+1)
     &           *VectorSphericalHarmonics(:,l(i),+1,m(i))
     &           -sqrt((l(i)+1)/(2*l(i)+DBLE(1.0)))*Bessel(l(i)-1)
     &           *VectorSphericalHarmonics(:,l(i),-1,m(i))
	   ENDIF
	  ENDIF
	 ENDDO
       ENDIF
       DEALLOCATE(l,m,magnetic)
       END SUBROUTINE GetBasis

!
! ****************************************************************** 
	SUBROUTINE Factorize(gridpointspCell,g1,g2,g3)
! ****************************************************************** 
	INTEGER,INTENT(in)::gridpointspCell
	INTEGER,INTENT(out)::g1,g2,g3
	SELECT CASE(gridpointspCell)
	  CASE(:3,5,7,11,13,17,19,23,29,31); g1=gridpointspCell;g2=1;g3=1          ! PRIMES
	  CASE(4); g1=2;g2=2;g3=1
	  CASE(6); g1=3;g2=2;g3=1
	  CASE(8); g1=2;g2=2;g3=2
	  CASE(9); g1=3;g2=3;g3=1
	  CASE(10); g1=5;g2=2;g3=1
	  CASE(12); g1=3;g2=2;g3=2
	  CASE(14); g1=7;g2=2;g3=1
	  CASE(15); g1=5;g2=3;g3=1
	  CASE(16); g1=4;g2=2;g3=2
	  CASE(18); g1=3;g2=3;g3=2
	  CASE(20); g1=5;g2=2;g3=2
	  CASE(21); g1=7;g2=3;g3=1
	  CASE(22); g1=11;g2=2;g3=1
	  CASE(24); g1=4;g2=3;g3=2
	  CASE(25); g1=5;g2=5;g3=1
	  CASE(26); g1=13;g2=2;g3=1
	  CASE(27); g1=3;g2=3;g3=3
	  CASE(28); g1=7;g2=2;g3=2
	  CASE(30); g1=5;g2=3;g3=2
	  CASE(32); g1=4;g2=4;g3=2
	  CASE DEFAULT
           write(*,*)'I am exhausted, please use smaller grid.'
           g1=gridpointspCell;g2=1;g3=1        
	END SELECT
	 IF (g1*g2*g3/=gridpointspCell) STOP 'Factorizer funnels'
	END SUBROUTINE Factorize


       END MODULE DefBaseGrid

!===========================================================================
!==========================================================================
