      MODULE Matrixia
      USE constants
      IMPLICIT none
      PUBLIC Solve, Matmul3, Matsqr3, Matmul3cc, Matmul3a
      CONTAINS

!***************************************************************************
      SUBROUTINE Solve(A,x,b,dim,dim2,method)
!***************************************************************************
      character(*),intent(in)::method
      integer,intent(in)::dim,dim2
      complex*16,intent(in),dimension(dim,dim)::A
      complex*16,intent(in),dimension(dim,dim2)::b
      complex*16,intent(out),dimension(dim,dim2)::x
      complex*16,ALLOCATABLE::AF(:,:),work(:)
      real*8,ALLOCATABLE::BERR(:),FERR(:),RWORK(:),S(:)
!
!complex,dimension(22*dim )::work
!
      INTEGER info ,ipiv(dim)  !,maxiter=3
      real*8 rcond
      character equed

      write(*,*)'Solver:',Method
        SELECT CASE (Method)
        CASE ('Lapack','CGESV') 
         x=b
!   CALL CGESV(dim,dim2,A,dim,ipiv,x,dim,info) 
! CGESVX also possible, more accurate, but slower  
        CASE ('LAPACK','ZGESV') 
         x=b
        CALL zgesv(dim,dim2,A,dim,ipiv,x,dim,info) 
! ZGESVX also possible, more accurate, but slower  
          IF (info /=0) write(*,*)'zgesv munii, info=',info
        CASE ('ZHESV') ! for complex hermitian
         x=b
         ALLOCATE(Work(22*dim))
!   CALL zhesv('Upper',dim,dim2,A,dim,ipiv,x,dim,work,22*dim,info) 
          IF (info /=0) write(*,*)'zhesv munii, info=',info
        CASE ('ZPOSV')  ! complex hermitian positive definite
         x=b
              CALL zposv('U',dim,dim2,A,dim,x,dim,info) 
          IF (info /=0) write(*,*)'zposv munii, info=',info
        CASE ('ZPOSVX')  ! complex hermitian positive definite
          ALLOCATE(AF(dim,dim),Work(2*dim),BERR(dim2),FERR(dim2),
     &             RWork(dim),S(dim))
          CALL zposvx('E','U',dim,dim2,A,dim,AF,dim,equed,S,b,dim,
     &             x,dim,rcond,ferr,berr,work,rwork,info) 
          IF (info /=0) write(*,*)'zposvx munii, info=',info
          write(*,*)'rcond',rcond
          write(*,*)'  max relative forward error',
     &            maxval(ferr),sum(ferr)/size(ferr)
          write(*,*)'     relative backward error',
     &            maxval(berr),sum(berr)/size(berr)
          DEALLOCATE(AF,Work,BERR,FERR,RWork,S)

        CASE DEFAULT
          x=DBLE(0.0)
        ENDSELECT

      END SUBROUTINE Solve

!***************************************************************************
      FUNCTION matmul3(A,B)
!***************************************************************************
!
!     matmul3=A*B    dim(A)=:,3     dim(B)=3,:
!
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A,B
       COMPLEX*16,DIMENSION(size(A,1),size(B,2))::matmul3
       INTEGER j
!
!  write(*,*)'matmul3',size(A,1),size(A,2),size(B,1),size(B,2)
!
        IF ((size(A,2)/=3) .OR. (size(B,1)/=3)) 
     &     STOP 'matmul3: wrong dimension'

        DO j=1,size(B,2)
          matmul3(:,j)=A(:,1)*B(1,j)+A(:,2)*B(2,j)+A(:,3)*B(3,j)
        ENDDO

       END FUNCTION matmul3



!***************************************************************************
      SUBROUTINE matmul3b(C,A,B)
!***************************************************************************
!
!     matmul3=A*B    dim(A)=:,3     dim(B)=3,:
!
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A,B
       COMPLEX*16,DIMENSION(size(A,1),size(B,2))::C
       INTEGER j
!
!  write(*,*)'matmul3',size(A,1),size(A,2),size(B,1),size(B,2)
!
        IF ((size(A,2)/=3) .OR. (size(B,1)/=3)) 
     &     STOP 'matmul3: wrong dimension'

        DO j=1,size(B,2)
          C(:,j)=A(:,1)*B(1,j)+A(:,2)*B(2,j)+A(:,3)*B(3,j)
        ENDDO

       END SUBROUTINE matmul3b
!***************************************************************************
      SUBROUTINE matmul3a(C,A,B)
!***************************************************************************
!
!     matmul3=A*B    dim(A)=:,3     dim(B)=3,:
!
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A,B
       COMPLEX*16,DIMENSION(size(A,1),size(B,2))::C
       INTEGER j
!
!  write(*,*)'matmul3',size(A,1),size(A,2),size(B,1),size(B,2)
!
        IF ((size(A,2)/=3) .OR. (size(B,1)/=3)) 
     &     STOP 'matmul3: wrong dimension'

        DO j=1,size(B,2)
          C(:,j)=C(:,j)+A(:,1)*B(1,j)+A(:,2)*B(2,j)+A(:,3)*B(3,j)
        ENDDO

       END SUBROUTINE matmul3a



!***************************************************************************
      SUBROUTINE matmul3cc(C,A,B)
!***************************************************************************
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::B
       COMPLEX*16,INTENT(inout),DIMENSION(:,:)::C
       INTEGER j
!
!      C=C+A*B+conj(transpose(A*B))
!
        IF ((size(A,2)/=3) .OR. (size(B,1)/=3))
     &          STOP 'matmul3: wrong dimension'

        DO j=1,size(B,2)
          C(:,j)=C(:,j)+     (A(:,1)*B(1,j)+A(:,2)*B(2,j)+A(:,3)*B(3,j)) 
          C(j,:)=C(j,:)+conjg(A(:,1)*B(1,j)+A(:,2)*B(2,j)+A(:,3)*B(3,j))

!              +conjg(A(j,1)*B(1,:)+A(j,2)*B(2,:)+A(j,3)*B(3,:))
        ENDDO
       END SUBROUTINE matmul3cc

!***************************************************************************
      FUNCTION matmul3dd(A,B)
!***************************************************************************
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::B
       COMPLEX*16,DIMENSION(size(A,1),size(B,2))::matmul3dd
       INTEGER j
!
!    matmul3dd=A*B+conj(transpose(A*B))
!
         write(*,*)'matmuldd3'
         write(*,*)size(A,1),size(A,2),size(B,1),size(B,2)
        IF ((size(A,2)/=3) .OR. (size(B,1)/=3)) 
     &           STOP 'matmul3: wrong dimension'

        DO j=1,size(B,2)
          matmul3dd(:,j)=A(:,1)*B(1,j)+A(:,2)*B(2,j)+A(:,3)*B(3,j)
     &           +conjg(A(j,1)*B(1,:)+A(j,2)*B(2,:)+A(j,3)*B(3,:))
        ENDDO
!
       END FUNCTION matmul3dd
!
!***************************************************************************
      SUBROUTINE matsqr3(C,A,b)  ! C=A'Ab
!***************************************************************************
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A
       COMPLEX*16,INTENT(inout),DIMENSION(:,:)::C
       REAL*4,INTENT(in)::b
       INTEGER j
!
!     C=C+conj(transpose(A))*A*b
!
        IF ((size(A,1)/=3)) STOP 'matsqr3: wrong dimension'
        DO j=1,size(A,2)
          C(:,j)=C(:,j)+conjg(A(1,:))*A(1,j)*b+conjg(A(2,:))*A(2,j)*b
     &      +conjg(A(3,:))*A(3,j)*b
        ENDDO
       END SUBROUTINE matsqr3

!***************************************************************************
      FUNCTION matsqr2(A,b)
!***************************************************************************
       COMPLEX*16,INTENT(in),DIMENSION(:,:)::A
       REAL*4,INTENT(in)::b
       COMPLEX*16,DIMENSION(size(A,1),size(A,1))::matsqr2
       INTEGER j
!
!     matspr2=b*conj(transpose(A))*A
!
        IF ((size(A,1)/=3)) STOP 'matsqr2: wrong dimension'
        DO j=1,size(A,2)
          matsqr2(:,j)=conjg(A(1,:))*A(1,j)*b+conjg(A(2,:))*A(2,j)*b
     &      +conjg(A(3,:))*A(3,j)*b
        ENDDO
       END FUNCTION matsqr2

      END MODULE Matrixia
