       module Scatterer
! this is a module to define the shape and optical properties
!  of the scatterer.

! Modify this for your own particles

! Note that particle coordinates and field coordinates are separate.
! You can thus define the particle in most comfort orientation.
! One yielding smallest surrounding box is numerically best.
! You can later rotate the world around Your Particle as you wish.


       USE Vectorix

       USE Constants
       USE Incidentia, ONLY: wavenumber

       IMPLICIT NONE

!      PUBLIC Suski,Permittivity,Chirality,Reciprocity,InitEnsamble,
!    &      InitScatterer,InParticle,RefractiveIndex


	TYPE grainmaintype      ! information for each grain
	  REAL*4,DIMENSION(3)::center
	  REAL*4,DIMENSION(3,3)::rotation
	  INTEGER,DIMENSION(6)::contactor
	  INTEGER class,ncontactors
	END TYPE grainmaintype

	TYPE grainclasstype       ! too much redundancy here
	  REAL*4,DIMENSION(3)::boxsize !minimum outer boundary cube
	  INTEGER shape
	  REAL*4,DIMENSION(3)::a,aa  ! shape dependent parameters
!                           sphere    a(1): radius (inner)
!                                     a(2): radius (outer)      
!                                    aa(1): radi**2
!                                    aa(2): rado**2
!                           ellipsoid a(.): semiaxises
!                                    aa(.): 1./semi's**2
!                           cube      a(1): side      
!                                    aa(1): side/2
	  LOGICAL homogeneous,isotropic,magnetic,chiral
	  COMPLEX*8 refindx
	  REAL*4 permi,chiraliti,reciprociti,minscale,maxscale
	! C2TENSOR::refindx
	END TYPE grainclasstype

	INTEGER,PARAMETER::spherical=0,ellipsoidical=1,
     &     sphericalhollow=2,cubical=3,rectangular=4,cylindrical=5,
     &     cylindricalhollow=6,spheroidicalCylindrical=7,neck=8,
     &     random=9
	INTEGER subgrains,   ! number of
     &          grainclasses,! 1:one subgrain 2:"some" subgrains
     &          OriAveType   ! oriental average type
	TYPE(grainmaintype),ALLOCATABLE,DIMENSION(:)::grain
	TYPE(grainclasstype),ALLOCATABLE,DIMENSION(:)::grainclass

	CONTAINS

!     *****************************************************************
      subroutine InitEnsamble(Configurations,Orientations,
     &            OriAveType)
!     *****************************************************************
! To be called once in the beginning to read input and preconfigure
!
!   This is an important user interface.
!   Thus it is left sufficiently confusing so that all idiots 
!   cannot find out how to use such a fine program.

!   In future, much more OPTIons will be given.



       INTEGER,INTENT(out),OPTIONAL::configurations,Orientations,
     & OriAveType
!
       INTEGER subgrain,subgrain2, ! number of subgrain to get dist/contacts
     &         no,totcon,          ! total number of contacts
     &         i,j
!
       CHARACTER(len=39) posifile  ! file consisting in geometrical data
!                                    on the subgrains' centers
       REAL*4 ren,imn,dd,
     &        dmax,dmin,        ! min,max distance between grains
     &        neckradius

       IF (PRESENT(OriAveType)) THEN
	Write(*,*)'How oriented? (0=as given,1=random isotropic,' 
        Write(*,*)'                          2=random in plane  )'
 	OriAveType=0
!	READ (*,*) OriAveType
       END IF
!
       IF (PRESENT(Orientations)) THEN
        WRITE(*,*) 'Number of orientations? (10-1000 are good numbers)'
!       Orientations=1
        READ (15,*) Orientations
        WRITE(*,*) Orientations,'orientations'
       END IF
!
       IF (PRESENT(Configurations)) THEN
	!       WRITE(*,*) 'Type of size averaging distribution?'
	!       WRITE(*,*) '0=none, 1=log-normal, 2=beta, 3=power,'
        !       WRITE(*,*) ' (only 0 works)'
	!       WRITE(*,*) 'Number of configurations to be averaged? '
        !       WRITE(*,*) '(only 1 makes sense yet)'
	!       READ (*,*) configurations
        configurations=1
        END IF
!
!- get number and position of subgrains
!
      WRITE (*,*) 'The number of subgrains? '
      subgrains=1
!     READ(*,*) subgrains
      grainclasses=min(subgrains,2)
      ALLOCATE(grain(subgrains*3),grainclass(grainclasses))  
      grain(:)%class=1
!
      IF (subgrains==1) THEN
	 WRITE (*,*) 'one subgrain'
	 grain(1)%center=0.0
	 grain(1)%ncontactors=0
	 grain(1)%rotation=0.0
         DO i=1,3; grain(1)%rotation(i,i)=REAL(1.0);ENDDO
!        WRITE (*,*)'rot',grain(1)%rotation
      ELSE
	 WRITE (*,*) subgrains,' (until changed, '
         WRITE (*,*)'every subgrain is identical)'
591      WRITE (*,*) 'Give file containing positions in format
     &                       : no, x,y,z'    
	 READ (*,*) posifile
	 OPEN (41,STATUS='old',FILE=posifile,ERR=591,ACTION='read')
	 DO subgrain=1,subgrains
	    READ(41,*) no,grain(subgrain)%center(1:3)
	    grain(subgrain)%center(1:3)=grain(subgrain)%center(1:3)
     &               *wavenumber
	    grain(subgrain)%ncontactors=0
	    grain(subgrain)%rotation=0.0
	    DO i=1,3; grain(subgrain)%rotation(i,i)=REAL(1.0);ENDDO
!        WRITE (*,*)'rot',grain(subgrain)%rotation
	 ENDDO
	 CLOSE(41)
	 WRITE(*,*) 'Distance between 1 and 2:',
     &   sqrt(sum((grain(1)%center-grain(2)%center)**2))/wavenumber
	 dmax=0;dmin=sum((grain(1)%center-grain(2)%center)**2)
	 DO subgrain=1,subgrains
	   DO subgrain2=subgrain+1,subgrains
	     dd=sum((grain(subgrain)%center
     &              -grain(subgrain2)%center)**2)
	     IF (dd>dmax) dmax=dd
	     IF (dd<dmin) dmin=dd
	   ENDDO
	 ENDDO
	 WRITE(*,*) 'Minimum distance:',sqrt(dmin)/wavenumber
         WRITE(*,*) '  Maximum:',sqrt(dmax)/wavenumber

      ENDIF

      IF (subgrains<1) STOP
      subgrain=1
!
!--- get shape of subgrains / all identical !!!
!
      WRITE(*,*)'Shape? '
      WRITE(*,*)'0=Spherical, 1=Ellipsoidial,
     & 2=Spherical hollow'
      WRITE(*,*)' 3=Cubical, 4=Rectangular, 5=Cylindrical,'
      WRITE(*,*)' 6=Cylindrical hollow, 9=random'
      READ(15,*) grainclass(1)%shape
      SELECT CASE (grainclass(1)%shape)
      CASE (Spherical) 
	 WRITE (*,*) 'Spherical particle. And what radius?'
	 READ (15,*) grainclass(1)%a(1)
	 WRITE (*,*) grainclass(1)%a(1)
	 grainclass(1)%a=grainclass(1)%a(1)*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize=2*grainclass(1)%a
	 grainclass(1)%minscale=grainclass(1)%a(1)
	 grainclass(1)%maxscale=grainclass(1)%minscale
	CASE (Ellipsoidical) 
	 WRITE (*,*) 'Ellipsoidial particle. Semiaxises (a,b,c)?'
	 READ (15,*) grainclass(1)%a(1:3)
	 WRITE (*,*) grainclass(1)%a(1:3)
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**(-2)
	 grainclass(1)%boxsize=2*grainclass(1)%a
	 grainclass(1)%minscale=minval(grainclass(1)%a)
	 grainclass(1)%maxscale=maxval(grainclass(1)%a)
	CASE (SphericalHollow) 
	 WRITE (*,*) 'Hollow Spherical particle.'
                WRITE (*,*) 'Inner and outer radius?'
	 READ (*,*) grainclass(1)%a(1:2)
	 WRITE (*,*) grainclass(1)%a(1:2)
	 grainclass(1)%a(3)=-grainclass(1)%a(1)+grainclass(1)%a(2) 
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize=2*grainclass(1)%a(2)
	 grainclass(1)%minscale=grainclass(1)%a(1)
                grainclass(1)%maxscale=grainclass(1)%a(2)
	CASE (cubical)
	 WRITE (*,*) 'Cubical particle. Side length?'
	 READ (15,*) grainclass(1)%a(1)
	 WRITE (*,*) grainclass(1)%a(1)
	 grainclass(1)%a=grainclass(1)%a(1)*wavenumber
	 grainclass(1)%aa=grainclass(1)%a/2
	 grainclass(1)%boxsize=grainclass(1)%a
	 grainclass(1)%maxscale=grainclass(1)%a(1)*sqrt(3.0)
	 grainclass(1)%minscale= grainclass(1)%maxscale*0.01
	CASE (rectangular)
	 WRITE (*,*) 'Rectangular particle. Sides?'
	 READ (15,*) grainclass(1)%a(1:3)
	 WRITE (*,*) grainclass(1)%a(1:3)
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize=grainclass(1)%a
	 grainclass(1)%minscale=0.0
         grainclass(1)%maxscale=sqrt(sum(grainclass(1)%a**2))/2
	CASE (Cylindrical) 
	 WRITE (*,*) 'Cylindrical particle. Radius, length?'
	 READ (15,*) grainclass(1)%a(1:3:2)
	 WRITE (*,*) grainclass(1)%a(1:3:2)
	 grainclass(1)%a(2)=grainclass(1)%a(1) 
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize(1:2)=2*grainclass(1)%a(1:2)
	 grainclass(1)%boxsize(3)=grainclass(1)%a(3)
	 grainclass(1)%minscale=0.0;grainclass(1)%maxscale=
     &         sqrt(sum(grainclass(1)%a(2:3)**2))/2
	CASE (SpheroidicalCylindrical) 
	 WRITE (*,*) 'Spheroidical Cylindrical particle. Axis_a,
     &          Axis_b, length?'
	 READ (15,*) grainclass(1)%a(1:3)
	 WRITE (*,*) grainclass(1)%a(1:3)
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize(1:2)=2*grainclass(1)%a(1:2)
	 grainclass(1)%boxsize(3)=grainclass(1)%a(3)
	 grainclass(1)%minscale=0.0
	 grainclass(1)%maxscale=sqrt(maxval(grainclass(1)%a(1:3))**2
     &          +grainclass(1)%a(3)**2)/2
	CASE (CylindricalHollow) 
	 WRITE(*,*)'Hollow Cylinder. Inner and outer Radius, Length?'
	 READ (*,*) grainclass(1)%a(1:3)
	 WRITE (*,*) grainclass(1)%a(1:3)
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize(1:2)=2*grainclass(1)%a(2)
	 grainclass(1)%boxsize(3)=grainclass(1)%a(3)
	 grainclass(1)%minscale=0.0
                grainclass(1)%maxscale=
     &              sqrt(sum(grainclass(1)%a(2:3)**2))/2
	CASE (Neck) 
	 WRITE (*,*) 'Neck (for the time being cutted hyberboloid).
     &             Min Radius, Other parameter, Third one?'
	 READ (*,*) grainclass(1)%a(1:3)
	 WRITE (*,*) grainclass(1)%a(1:3)
	 grainclass(1)%a=grainclass(1)%a*wavenumber
	 grainclass(1)%aa=REAL(1.0)/grainclass(1)%a(1)**2
!        grainclass(1)%boxsize(1:2)=grainclass(1)%a(1)/seedclass%a(1)*
!    &   sqrt(2*seedclass%a(1)**2-neckradius**2)
	 grainclass(1)%boxsize=0.0
	 grainclass(1)%minscale=minval(grainclass(1)%a(1:2))
	 grainclass(1)%maxscale=maxval(grainclass(1)%boxsize/2)
	CASE (random) 
	 WRITE (*,*) 'Random. Mean radius, standard deviation,
     &           correlation length [rad]?'
	 READ (*,*) grainclass(1)%a(1:3)
	 WRITE (*,*) grainclass(1)%a(1:3)
	 grainclass(1)%a(1:2)=grainclass(1)%a(1:2)*wavenumber
	 grainclass(1)%aa=grainclass(1)%a**2
	 grainclass(1)%boxsize=2*(grainclass(1)%a(1)
     &          +3.3*grainclass(1)%a(2))
	CASE DEFAULT
	 stop 'not working'
	END SELECT
!
!--- multiple subgrains : get contactations
!
	IF (subgrains>1) THEN 
	 totcon=0
	 WRITE (*,*) 'Neck radius? (0 if no)'
	 READ (*,*) Neckradius
	 IF (NeckRadius>REAL(0.0)) THEN
!
	   CALL CreateConClass(grainclass(2),
     &           Neckradius*wavenumber,grainclass(1))
!
	   grain(:)%ncontactors=0
	   grain(:)%ncontactors=0
!
	   DO subgrain=1,subgrains
	     DO subgrain2=1,subgrain-1
!               subgrain,subgrain2
!
	       IF (inContact(subgrain,subgrain2,
     &                   Neckradius*wavenumber)) THEN
		 grain(subgrain)%ncontactors=
     &                     grain(subgrain)%ncontactors+1
		 grain(subgrain2)%ncontactors=
     &                  grain(subgrain2)%ncontactors+1
		 totcon=totcon+1
		 grain(subgrain)%contactor(grain(subgrain)
     &                  %ncontactors)=subgrains+totcon
		 grain(subgrain2)%contactor(grain(subgrain2)
     &                  %ncontactors)=subgrains+totcon
!
		 CALL CreateContactor(grain(subgrain),
     &                  grain(subgrain2),grain(totcon+subgrains))
!
!              ELSE
!                 WRITE (*,*)'nocontact'
	       ENDIF
!
	     ENDDO
	   ENDDO
	   subgrains=subgrains+totcon
	 ENDIF
       ENDIF
!      WRITE (*,*)'Inner structure: 0=homogeneous, 
!                   1=stochastically inhomogeneous'
!      WRITE (*,*) 'Optical properties data file. Select from:'
!      WRITE (*,*) 'None, {water, ice, silicate, ...}[.opd], 
!                       or /your/own/subdirectory/file.name'
!      READ (*,*) optidata
!      IF (optidata==''.OR.optidata=='None') THEN
 45      WRITE (*,*) 'Refractive index: real part, imaginary part ?'
       	READ (15,*)  ren,imn
	grainclass(:)%refindx=ren+im*imn
!      ELSE
!      OPEN (44,ERR=45,IOSTAT=ios,FILE=optidata)
!      READ (44,  )
!      ENDIF
       WRITE (*,*) grainclass(1)%refindx

      END SUBROUTINE InitEnsamble


!     ****************************************************
       SUBROUTINE  CreateContactor(grain1,grain2,congrain)
!     ****************************************************
       TYPE(grainmaintype),INTENT(in)::grain1,grain2
       TYPE(grainmaintype),INTENT(out)::congrain
	       
       congrain%center=(grain1%center+grain2%center)*0.5
       congrain%rotation(:,3)=(grain1%center-grain2%center)
     & /sqrt(sum((grain1%center-grain2%center)**2))
       IF ((congrain%rotation(2,3)==REAL(0.0)).AND.
     &        (congrain%rotation(1,3)==REAL(0.0))) THEN
	congrain%rotation(:,2)=Normalize((/-REAL(1.0),
     &                         REAL(0.0),REAL(0.0)/))
       ELSE
	congrain%rotation(:,2)=Normalize((/-congrain%rotation(2,3),
     &   congrain%rotation(1,3),REAL(0.0)/))
       ENDIF
       congrain%rotation(:,1)=Normalize(congrain
     &      %rotation(:,2).cross.congrain%rotation(:,3))
       congrain%rotation=transpose(congrain%rotation)
       congrain%class=2
       congrain%ncontactors=0
       END SUBROUTINE  CreateContactor

!     ****************************************************
       SUBROUTINE  CreateConClass(conclass,neckradius,seedclass)
!     ****************************************************
       TYPE(grainclasstype),INTENT(out)::conclass       
       TYPE(grainclasstype),INTENT(in)::seedclass
       REAL*4,INTENT(in)::neckradius
	      
       conclass=seedclass
       conclass%shape=Neck
       conclass%aa(1)=REAL(1.0)/neckradius**2
       conclass%aa(2)=(seedclass%a(1)/neckradius)**2-REAL(1.0)
       conclass%aa(3)=(seedclass%a(1)**2-neckradius**2)/
     &      (neckradius**2*(2*seedclass%a(1)**2-neckradius**2))
       conclass%a=REAL(1.0)/sqrt(conclass%aa(1))
       conclass%boxsize(1:2)=Neckradius/seedclass%a(1)
     &      *sqrt(2*seedclass%a(1)**2-neckradius**2)
       conclass%boxsize(3)=2*Neckradius
       conclass%minscale=minval(conclass%a)
       conclass%maxscale=maxval(conclass%boxsize/2)
       END SUBROUTINE  CreateConClass


!     ****************************************
       subroutine InitScatterer(conf)
!     ****************************************
! To be called for each configuration when configuration averaging

       INTEGER,INTENT(in)::conf
	  IF (conf==44) write(*,*)conf
       END SUBROUTINE InitScatterer

!**********************************************************************
       RECURSIVE function InParticle(r,subgrain) RESULT (Ingrain)
!**********************************************************************
!   This is the interface between particle and scattering.
!   Called many times by other modules:
!                           by the "material routines"
!                           gridizer.f by "GenerateGrid" 
!   You may want to modify this.
!   Usually not the most speed critical, but still, don't complicate too much.
!
       LOGICAL InGrain
!
       TYPE(grainmaintype),INTENT(in)::subgrain   !grain information
!
       REAL*4,DIMENSION(3), intent(in)::r         !grain center/position
       REAL*4,DIMENSION(3)::rc
!
       INTEGER con
!
!
	IF (grainclass(subgrain%class)%shape==spherical) THEN
	 rc=r-subgrain%center           ! symmetric anyway, isn't it?
	ELSE
           rc=subgrain%rotation(:,1)*(r(1)-subgrain%center(1))
     &        +subgrain%rotation(:,2)*(r(2)-subgrain%center(2))
     &        +subgrain%rotation(:,3)*(r(3)-subgrain%center(3)) 
	ENDIF
!        IF (ANY(abs(rc)>grainclass(subgrain%class)%boxsize*0.5)) THEN
!          InGrain=.FALSE.;RETURN
!        ENDIF
!
	SELECT CASE(grainclass(subgrain%class)%shape)
!
	  CASE (spherical)    ! dr**2 < a**2
	   IF (dot_product(rc,rc)>grainclass(subgrain%class)%aa(1)) THEN 
	     InGrain=.FALSE.
	   ELSE  ! point inside the scatterer
	     InGrain=.TRUE.
	   ENDIF
!
	  CASE (Sphericalhollow)     ! ai**2 < dr**2 < ao**2
	   IF ((dot_product(rc,rc)>grainclass(subgrain%class)%aa(2)) .OR.
     &       (dot_product(rc,rc)<grainclass(subgrain%class)%aa(1))) THEN 
	     InGrain=.FALSE.
	   ELSE  ! point inside the scatterer
	     InGrain=.TRUE.
	   ENDIF
!
	  CASE (ellipsoidical)       ! (x/a)**2+(y/b)**2+(z/c)**2 < 1.
	   IF (dot_product(rc,rc*grainclass(subgrain%class)%aa)>REAL(1.0))
     &       THEN 
	     InGrain=.FALSE.
	   ELSE  ! point inside the scatterer
	    InGrain=.TRUE.
	   ENDIF
!
	  CASE (cubical,rectangular)  ! dr < side/2
	   IF (ANY(rc<-grainclass(subgrain%class)%aa) .OR.
     &       ANY(rc>grainclass(subgrain%class)%aa)) THEN
	    InGrain=.FALSE.
	   ELSE  ! point inside the scatterer
	    InGrain=.TRUE.
	   ENDIF
!
	  CASE (cylindrical) ! x**2+y**2 < r**2 and z < h
	   IF ((sum(rc(1:2)**2)>grainclass(subgrain%class)%aa(1))
     &      .OR.(rc(3)<grainclass(subgrain%class)%aa(3)) 
     &      .OR.(rc(3)>grainclass(subgrain%class)%aa(3))) THEN 
	     InGrain=.FALSE.
	   ELSE  ! point inside the scatterer
	     InGrain=.TRUE.
	   ENDIF
!
	  CASE (cylindricalhollow)
	    IF ((sum(rc(1:2)**2)>grainclass(subgrain%class)%aa(2))
     &    .OR.(sum(rc(1:2)**2)<grainclass(subgrain%class)%aa(1))
     &    .OR.(rc(3)<grainclass(subgrain%class)%aa(3))
     &    .OR.(rc(3)>grainclass(subgrain%class)%aa(3))) THEN 
              InGrain=.FALSE.
	    ELSE  ! point inside the scatterer
	      InGrain=.TRUE.
	    ENDIF
!
	  CASE (Neck)
	   IF ((grainclass(subgrain%class)%aa(1)*sum(rc(1:2)**2) 
     & -grainclass(subgrain%class)%aa(2)*rc(3)**2) >REAL(1.0)) THEN
	     InGrain=.FALSE.
	   ELSEIF ((grainclass(subgrain%class)%aa(3)*sum(rc(1:2)**2) 
     &   +grainclass(subgrain%class)%aa(1)*rc(3)**2)  >REAL(1.0)) THEN
	     InGrain=.FALSE.
	   ELSE  ! point inside the scatterer
	     InGrain=.TRUE.
	   ENDIF
!
	  CASE DEFAULT
	    STOP 'InParticle'
!
	 ENDSELECT
!
!-- check contact areas
!
	 IF (InGrain) THEN
	   DO con=1,subgrain%ncontactors
	     IF (InParticle(r,grain(subgrain%contactor(con)))) THEN
	       InGrain=.FALSE.;EXIT
	     ENDIF
	   ENDDO
	 ENDIF
!
       END FUNCTION InParticle

!     *****************************************************************
       LOGICAL FUNCTION InContact(grain1,grain2,contactdistance)
!     *****************************************************************
       INTEGER,INTENT(in)::grain1,grain2
       REAL*4,INTENT(in)::contactdistance
       IF ((Grainclass(grain(grain1)%class)%shape==spherical) .AND. 
     &  (Grainclass(grain(grain2)%class)%shape==spherical) .AND. 
     &  (sum((grain(grain1)%center-grain(grain2)%center)**2) < 
     &  (contactdistance+grainclass(grain(grain1)%class)%a(1)
     &  +grainclass(grain(grain2)%class)%a(1))**2)) THEN
	 InContact=.TRUE.
       ELSE
	InContact=.FALSE.
       ENDIF
       END FUNCTION InContact

!     *****************************************************************
!      function RefractiveIndex(rvec,subgrain)
!     *****************************************************************
!   This is the interface between particle and scattering.
!   Called many times by other modules.
!   You may want to modify this.
!
!      complex*8 RefractiveIndex
!      REAL*4,DIMENSION(3), intent(in),OPTIONAL::rvec
!      INTEGER,INTENT(in),OPTIONAL::subgrain
!      INTEGER graini
!
!      IF (PRESENT(subgrain).AND..NOT.PRESENT(rvec)) THEN
!           RefractiveIndex=grainclass(grain(subgrain)%class)%refindx
!      ELSE IF (PRESENT(subgrain).AND.PRESENT(rvec)) THEN
!           IF (InParticle(rvec,grain(subgrain))) THEN  ! point inside
!              RefractiveIndex=grainclass(grain(subgrain)%class)%refindx
!           ELSE  ! point in free space
!              RefractiveIndex=z1
!           ENDIF
!      ELSE IF (PRESENT(rvec)) THEN  
!           DO graini=1,subgrains
!             IF (InParticle(rvec,grain(graini))) THEN  
!              RefractiveIndex=grainclass(grain(graini)%class)%refindx
!                EXIT
!             ELSE
!                RefractiveIndex=z1
!             ENDIF
!           ENDDO
!      ELSE  ! point in free space
!           RefractiveIndex=z1
!      ENDIF
!      END FUNCTION RefractiveIndex
!
!  *****************************************************************
!      FUNCTION Suski(r,subgrain)
!  *****************************************************************
!      complex*8 Suski
!      REAL*4,DIMENSION(3), intent(in)::r
!      INTEGER,INTENT(in)::subgrain
!
!      IF (InParticle(r,grain(subgrain))) THEN  ! point inside
!           Suski=grainclass(grain(subgrain)%class)%refindx**2-1.0
!      ELSE  ! point in free space
!           Suski=0.0
!      ENDIF
!      END FUNCTION Suski
!
!  *****************************************************************
!      FUNCTION Permittivity(r,subgrain)
!  *****************************************************************
!      complex*8 Permittivity
!      REAL*4,DIMENSION(3), intent(in)::r
!      INTEGER,INTENT(in)::subgrain
!
!      IF (InParticle(r,grain(subgrain))) THEN  ! point inside
!           Permittivity=grainclass(grain(subgrain)%class)%permi
!      ELSE  ! point in free space
!           Permittivity=1.0
!      ENDIF
!      END FUNCTION Permittivity
!
!  *****************************************************************
       FUNCTION Chirality(r,subgrain)
!  *****************************************************************
       complex*8 Chirality
       REAL*4,DIMENSION(3), intent(in)::r
       INTEGER,INTENT(in)::subgrain

       IF (InParticle(r,grain(subgrain))) THEN  ! point inside
            Chirality=grainclass(grain(subgrain)%class)%Chiraliti
       ELSE  ! point in free space
            Chirality=0.0
       ENDIF
       END FUNCTION Chirality

!  *****************************************************************
       FUNCTION Reciprocity(r,subgrain)
!  *****************************************************************
       complex*8 Reciprocity
       REAL*4,DIMENSION(3), intent(in)::r
       INTEGER,INTENT(in)::subgrain

       IF (InParticle(r,grain(subgrain))) THEN  ! point inside
            Reciprocity=grainclass(grain(subgrain)%class)%reciprociti
       ELSE  ! point in free space
            Reciprocity=0.0
       ENDIF
       END FUNCTION Reciprocity

       END MODULE Scatterer


